package top.ninwoo.utils.service.impl;

import com.spotify.docker.client.messages.Container;
import com.spotify.docker.client.messages.ContainerInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import top.ninwoo.utils.entity.DockerContainer;
import top.ninwoo.utils.service.DockerService;
import top.ninwoo.utils.util.DockerUtils;
import top.ninwoo.utils.util.LinuxCtlUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Service
public class DockerServiceImpl implements DockerService {

    @Autowired
    private LinuxCtlUtils linuxCtlUtils;

    @Autowired
    private DockerUtils dockerUtils;

    // 这里考虑要不要区分正在运行中的容器
    private ConcurrentHashMap<String, DockerContainer> containersMap = new ConcurrentHashMap<>();

    @Override
    public List<String> getDockerIds() {
        return getDockerIds(false);
    }
    /**
     * 获取容器Id
     * @param isAll 是否获取全部的容器id,包含未启动的容器
     * @return
     */
    @Override
    public List<String> getDockerIds(boolean isAll) {
        return new ArrayList<String>(containersMap.keySet());
    }

    /**
     * 将DockerClient获取的到结果封装成为我们自定义的数据结构
     * @param containers
     * @return
     */
    public List<DockerContainer> convertDockerResult(List<Container> containers) {
        if(containers == null) {
            throw new RuntimeException("容器列表不能为null");
        }
        List<DockerContainer> dockerContainers = new ArrayList<DockerContainer>(containers.size());
        containers.forEach(c -> {
            DockerContainer container = new DockerContainer();
            container.setId(c.id());
            container.setCommand(c.command());
            container.setCreated(c.created());
            container.setPorts(c.portsAsString());
            container.setImage(c.image());
            container.setStatus(c.status());
            container.setName(c.names().get(0));
            dockerContainers.add(container);
        });
        return dockerContainers;
    }

    @Override
    public void getContainers() {
        updateContainers(true);
    }

    /**
     * @description 更新容器列表
     * @param isfull true:全量更新
     * @date 2019-10-20
     * 修改该服务类为完全无状态的类，容器信息由上层集群服务维护 修改了
     * @see DockerServiceImpl#updateContainers(boolean)
     * @see DockerServiceImpl#updateContainers()
     * @return
     */
    @Override
    public Map<String, DockerContainer> getContainers(boolean isfull) {
        if(containersMap.isEmpty()) {
            return dockerUtils.getContainers();
        } else {
            // TODO: 先执行增量更新，在返回结果
            return dockerUtils.getContainers();
        }
    }

    /**
     * 使用系统命令 docker run -itd 在后台启动一个容器,
     * 使用这种方式的一个重要愿意,是因为我没搞明白docker java启动容器的api到底如何使用
     * @param container 输入一个容器的描述信息
     * @return 返回容器启动之后真实的描述信息
     */
    @Override
    public DockerContainer runDocker(DockerContainer container) {
        container = dockerUtils.runDocker(container);
        return container;
    }

    @Override
    public DockerContainer getDockerById(String id) {
        return dockerUtils.getDockerById(id);
    }

    public DockerContainer exchangeFromDockerInfo(ContainerInfo containerInfo) {
        if(containerInfo == null) {
            throw new RuntimeException("ContainerInfo cannot be null");
        }
        DockerContainer dockerContainer = new DockerContainer();
        dockerContainer.setImage(containerInfo.image());
        dockerContainer.setName(containerInfo.name());
        dockerContainer.setId(containerInfo.id());
        dockerContainer.setStatus(containerInfo.state().status());
        dockerContainer.setCreated(containerInfo.created().getTime());
        return dockerContainer;
    }

    @Override
    public boolean deleteDockerById(String id) {
        return dockerUtils.deleteDockerById(id);
    }

    @Override
    public void updateContainers() {
        updateContainers(true);
    }

    @Override
    public void updateContainers(boolean isfull) {
        if(isfull) {
            // 清空
            containersMap.clear();
            List<Container> containers =
                    new ArrayList<Container>();

            containersMap.putAll(dockerUtils.getContainers());
        } else {
            // TODO: 增量备份
        }
    }
}
